@@ -1,6 +1,6 @@
 /*
  * PostGIS Raster - Raster Types for PostGIS
- * http://www.postgis.org/support/wiki/index.php?WKTRasterHomePage
+ * http://trac.osgeo.org/postgis/wiki/WKTRaster
  *
  * Copyright (C) 2012 Regents of the University of California
  *   <bkpark@ucdavis.edu>
@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -29,7 +29,7 @@ static void test_gdal_configured() {
 }
 
 static void test_gdal_drivers() {
-	int i;
+	uint32_t i;
 	uint32_t size;
 	rt_gdaldriver drv = NULL;
 
@@ -67,7 +67,9 @@ static void test_gdal_rasterize() {
 	wkb_len = (int) ceil(((double) strlen(wkb_hex)) / 2);
 	wkb = (unsigned char *) rtalloc(sizeof(unsigned char) * wkb_len);
 	for (i = 0; i < wkb_len; i++) {
-		sscanf(pos, "%2hhx", &wkb[i]);
+		int b = 0;
+		sscanf(pos, "%2x", &b);
+		wkb[i] = (unsigned char)b;
 		pos += 2;
 	}
 
@@ -96,16 +98,6 @@ static void test_gdal_rasterize() {
 	cu_free_raster(raster);
 }
 
-static char *
-lwgeom_to_text(const LWGEOM *lwgeom) {
-	char *wkt;
-	size_t wkt_size;
-
-	wkt = lwgeom_to_wkt(lwgeom, WKT_ISO, DBL_DIG, &wkt_size);
-
-	return wkt;
-}
-
 static rt_raster fillRasterToPolygonize(int hasnodata, double nodataval) {
 	rt_band band = NULL;
 	rt_pixtype pixtype = PT_32BF;
@@ -125,7 +117,7 @@ static rt_raster fillRasterToPolygonize(int hasnodata, double nodataval) {
 		for (x = 0; x < rt_band_get_width(band); ++x)
 			for (y = 0; y < rt_band_get_height(band); ++y)
 				rt_band_set_pixel(band, x, y, 0.0, NULL);
-	} 
+	}
 
 	rt_band_set_pixel(band, 3, 1, 1.8, NULL);
 	rt_band_set_pixel(band, 4, 1, 1.8, NULL);
@@ -164,7 +156,9 @@ static void test_gdal_polygonize() {
 	rt_raster rt;
 	int nPols = 0;
 	rt_geomval gv = NULL;
-	char *wkt = NULL;
+	LWGEOM *gexpected, *gobserved;
+	gexpected = lwgeom_from_wkt("POLYGON((3 1,3 2,2 2,2 3,1 3,1 6,2 6,2 7,3 7,3 8,5 8,5 6,3 6,3 3,4 3,5 3,5 1,3 1))",
+				   LW_PARSER_CHECK_NONE);
 
 	rt = fillRasterToPolygonize(1, -1.0);
 	CU_ASSERT(rt_raster_has_band(rt, 0));
@@ -172,46 +166,36 @@ static void test_gdal_polygonize() {
 	nPols = 0;
 	gv = rt_raster_gdal_polygonize(rt, 0, TRUE, &nPols);
 
-#ifdef GDALFPOLYGONIZE
 	CU_ASSERT_DOUBLE_EQUAL(gv[0].val, 1.8, FLT_EPSILON);
-#else
-	CU_ASSERT_DOUBLE_EQUAL(gv[0].val, 2.0, 1.);
-#endif
 
-	wkt = lwgeom_to_text((const LWGEOM *) gv[0].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((3 1,3 2,2 2,2 3,1 3,1 6,2 6,2 7,3 7,3 8,5 8,5 6,3 6,3 3,4 3,5 3,5 1,3 1))");
-	rtdealloc(wkt);
+	gobserved = (LWGEOM *)gv[0].geom;
+
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
 	CU_ASSERT_DOUBLE_EQUAL(gv[1].val, 0.0, FLT_EPSILON);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[1].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((3 3,3 6,6 6,6 3,3 3))");
-	rtdealloc(wkt);
+	gobserved = (LWGEOM *)gv[1].geom;
+	gexpected = lwgeom_from_wkt("POLYGON((3 3,3 6,6 6,6 3,3 3))", LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON );
 
-#ifdef GDALFPOLYGONIZE
 	CU_ASSERT_DOUBLE_EQUAL(gv[2].val, 2.8, FLT_EPSILON);
-#else
-	CU_ASSERT_DOUBLE_EQUAL(gv[2].val, 3.0, 1.);
-#endif
-
-	wkt = lwgeom_to_text((const LWGEOM *) gv[2].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((5 1,5 3,6 3,6 6,5 6,5 8,6 8,6 7,7 7,7 6,8 6,8 3,7 3,7 2,6 2,6 1,5 1))");
-	rtdealloc(wkt);
+	gobserved = (LWGEOM *)gv[2].geom;
+	gexpected = lwgeom_from_wkt("POLYGON((5 1,5 3,6 3,6 6,5 6,5 8,6 8,6 7,7 7,7 6,8 6,8 3,7 3,7 2,6 2,6 1,5 1))",
+				    LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
+	gobserved = (LWGEOM *)gv[3].geom;
+	gexpected = lwgeom_from_wkt(
+	    "POLYGON((0 0,0 9,9 9,9 0,0 0),(6 7,6 8,3 8,3 7,2 7,2 6,1 6,1 3,2 3,2 2,3 2,3 1,6 1,6 2,7 2,7 3,8 3,8 6,7 6,7 7,6 7))",
+	    LW_PARSER_CHECK_NONE);
 	CU_ASSERT_DOUBLE_EQUAL(gv[3].val, 0.0, FLT_EPSILON);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[3].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((0 0,0 9,9 9,9 0,0 0),(6 7,6 8,3 8,3 7,2 7,2 6,1 6,1 3,2 3,2 2,3 2,3 1,6 1,6 2,7 2,7 3,8 3,8 6,7 6,7 7,6 7))");
-	rtdealloc(wkt);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
 	for (i = 0; i < nPols; i++) lwgeom_free((LWGEOM *) gv[i].geom);
 	rtdealloc(gv);
 	cu_free_raster(rt);
 
 	/* Second test: NODATA value = 1.8 */
-#ifdef GDALFPOLYGONIZE
 	rt = fillRasterToPolygonize(1, 1.8);
-#else
-	rt = fillRasterToPolygonize(1, 2.0);
-#endif
 
 	/* We can check rt_raster_has_band here too */
 	CU_ASSERT(rt_raster_has_band(rt, 0));
@@ -227,48 +211,36 @@ static void test_gdal_polygonize() {
 	}
 	*/
 
-#ifdef GDALFPOLYGONIZE
 	CU_ASSERT_DOUBLE_EQUAL(gv[1].val, 0.0, FLT_EPSILON);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[1].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((3 3,3 6,6 6,6 3,3 3))");
-	rtdealloc(wkt);
+	gobserved = (LWGEOM *)gv[1].geom;
+	gexpected = lwgeom_from_wkt("POLYGON((3 3,3 6,6 6,6 3,3 3))", LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
+	//wkt = lwgeom_to_text((const LWGEOM *) gv[1].geom);
+	//CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((3 3,3 6,6 6,6 3,3 3))");
+	//rtdealloc(wkt);
 
 	CU_ASSERT_DOUBLE_EQUAL(gv[2].val, 2.8, FLT_EPSILON);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[2].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((5 1,5 3,6 3,6 6,5 6,5 8,6 8,6 7,7 7,7 6,8 6,8 3,7 3,7 2,6 2,6 1,5 1))");
-	rtdealloc(wkt);
+	gobserved = (LWGEOM *)gv[2].geom;
+	gexpected = lwgeom_from_wkt("POLYGON((5 1,5 3,6 3,6 6,5 6,5 8,6 8,6 7,7 7,7 6,8 6,8 3,7 3,7 2,6 2,6 1,5 1))",
+				    LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
+	//wkt = lwgeom_to_text((const LWGEOM *) gv[2].geom);
+	//CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((5 1,5 3,6 3,6 6,5 6,5 8,6 8,6 7,7 7,7 6,8 6,8 3,7 3,7 2,6 2,6 1,5 1))");
+	//rtdealloc(wkt);
 
 	CU_ASSERT_DOUBLE_EQUAL(gv[3].val, 0.0, FLT_EPSILON);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[3].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((0 0,0 9,9 9,9 0,0 0),(6 7,6 8,3 8,3 7,2 7,2 6,1 6,1 3,2 3,2 2,3 2,3 1,6 1,6 2,7 2,7 3,8 3,8 6,7 6,7 7,6 7))");
-	rtdealloc(wkt);
-#else
-	CU_ASSERT_DOUBLE_EQUAL(gv[0].val, 0.0, 1.);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[0].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((3 3,3 6,6 6,6 3,3 3))");
-	rtdealloc(wkt);
-
-	CU_ASSERT_DOUBLE_EQUAL(gv[1].val, 3.0, 1.);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[1].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((5 1,5 3,6 3,6 6,5 6,5 8,6 8,6 7,7 7,7 6,8 6,8 3,7 3,7 2,6 2,6 1,5 1))");
-	rtdealloc(wkt);
-
-	CU_ASSERT_DOUBLE_EQUAL(gv[2].val, 0.0, 1.);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[2].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((0 0,0 9,9 9,9 0,0 0),(6 7,6 8,3 8,3 7,2 7,2 6,1 6,1 3,2 3,2 2,3 2,3 1,6 1,6 2,7 2,7 3,8 3,8 6,7 6,7 7,6 7))");
-	rtdealloc(wkt);
-#endif
+	gobserved = (LWGEOM *)gv[3].geom;
+	gexpected = lwgeom_from_wkt(
+	    "POLYGON((0 0,0 9,9 9,9 0,0 0),(6 7,6 8,3 8,3 7,2 7,2 6,1 6,1 3,2 3,2 2,3 2,3 1,6 1,6 2,7 2,7 3,8 3,8 6,7 6,7 7,6 7))",
+	    LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
 	for (i = 0; i < nPols; i++) lwgeom_free((LWGEOM *) gv[i].geom);
 	rtdealloc(gv);
 	cu_free_raster(rt);
 
 	/* Third test: NODATA value = 2.8 */
-#ifdef GDALFPOLYGONIZE
 	rt = fillRasterToPolygonize(1, 2.8);
-#else	
-	rt = fillRasterToPolygonize(1, 3.0);
-#endif
 
 	/* We can check rt_raster_has_band here too */
 	CU_ASSERT(rt_raster_has_band(rt, 0));
@@ -284,30 +256,22 @@ static void test_gdal_polygonize() {
 	}
 	*/
 
-#ifdef GDALFPOLYGONIZE
-	CU_ASSERT_DOUBLE_EQUAL(gv[0].val, 1.8, FLT_EPSILON);
-
 	CU_ASSERT_DOUBLE_EQUAL(gv[3].val, 0.0, FLT_EPSILON);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[3].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((0 0,0 9,9 9,9 0,0 0),(6 7,6 8,3 8,3 7,2 7,2 6,1 6,1 3,2 3,2 2,3 2,3 1,6 1,6 2,7 2,7 3,8 3,8 6,7 6,7 7,6 7))");
-	rtdealloc(wkt);
-#else
-	CU_ASSERT_DOUBLE_EQUAL(gv[0].val, 2.0, 1.);
+	gobserved = (LWGEOM *)gv[3].geom;
+	gexpected = lwgeom_from_wkt(
+	    "POLYGON((0 0,0 9,9 9,9 0,0 0),(6 7,6 8,3 8,3 7,2 7,2 6,1 6,1 3,2 3,2 2,3 2,3 1,6 1,6 2,7 2,7 3,8 3,8 6,7 6,7 7,6 7))",
+	    LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
-	CU_ASSERT_DOUBLE_EQUAL(gv[2].val, 0.0, 1.);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[2].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((0 0,0 9,9 9,9 0,0 0),(6 7,6 8,3 8,3 7,2 7,2 6,1 6,1 3,2 3,2 2,3 2,3 1,6 1,6 2,7 2,7 3,8 3,8 6,7 6,7 7,6 7))");
-	rtdealloc(wkt);
-#endif
-
-	wkt = lwgeom_to_text((const LWGEOM *) gv[0].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((3 1,3 2,2 2,2 3,1 3,1 6,2 6,2 7,3 7,3 8,5 8,5 6,3 6,3 3,4 3,5 3,5 1,3 1))");
-	rtdealloc(wkt);
+	gobserved = (LWGEOM *)gv[0].geom;
+	gexpected = lwgeom_from_wkt(
+	    "POLYGON((3 1,3 2,2 2,2 3,1 3,1 6,2 6,2 7,3 7,3 8,5 8,5 6,3 6,3 3,4 3,5 3,5 1,3 1))", LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
 	CU_ASSERT_DOUBLE_EQUAL(gv[1].val, 0.0, FLT_EPSILON);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[1].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((3 3,3 6,6 6,6 3,3 3))");
-	rtdealloc(wkt);
+	gobserved = (LWGEOM *)gv[1].geom;
+	gexpected = lwgeom_from_wkt("POLYGON((3 3,3 6,6 6,6 3,3 3))", LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
 	for (i = 0; i < nPols; i++) lwgeom_free((LWGEOM *) gv[i].geom);
 	rtdealloc(gv);
@@ -320,7 +284,7 @@ static void test_gdal_polygonize() {
 
 	nPols = 0;
 	gv = rt_raster_gdal_polygonize(rt, 0, TRUE, &nPols);
-	
+
 	/*
 	for (i = 0; i < nPols; i++) {
 		wkt = lwgeom_to_text((const LWGEOM *) gv[i].geom);
@@ -329,25 +293,16 @@ static void test_gdal_polygonize() {
 	}
 	*/
 
-#ifdef GDALFPOLYGONIZE
 	CU_ASSERT_DOUBLE_EQUAL(gv[0].val, 1.8, FLT_EPSILON);
-#else
-	CU_ASSERT_DOUBLE_EQUAL(gv[0].val, 2.0, 1.);
-#endif
+	gobserved = (LWGEOM *)gv[0].geom;
+	gexpected = lwgeom_from_wkt("POLYGON((3 1,3 2,2 2,2 3,1 3,1 6,2 6,2 7,3 7,3 8,5 8,5 6,3 6,3 3,4 3,5 3,5 1,3 1))", LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
-	wkt = lwgeom_to_text((const LWGEOM *) gv[0].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((3 1,3 2,2 2,2 3,1 3,1 6,2 6,2 7,3 7,3 8,5 8,5 6,3 6,3 3,4 3,5 3,5 1,3 1))");
-	rtdealloc(wkt);
-
-#ifdef GDALFPOLYGONIZE
 	CU_ASSERT_DOUBLE_EQUAL(gv[1].val, 2.8, FLT_EPSILON);
-#else
-	CU_ASSERT_DOUBLE_EQUAL(gv[1].val, 3.0, 1.);
-#endif
-
-	wkt = lwgeom_to_text((const LWGEOM *) gv[1].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((5 1,5 3,6 3,6 6,5 6,5 8,6 8,6 7,7 7,7 6,8 6,8 3,7 3,7 2,6 2,6 1,5 1))");
-	rtdealloc(wkt);
+	gobserved = (LWGEOM *)gv[1].geom;
+	gexpected = lwgeom_from_wkt("POLYGON((5 1,5 3,6 3,6 6,5 6,5 8,6 8,6 7,7 7,7 6,8 6,8 3,7 3,7 2,6 2,6 1,5 1))",
+				    LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
 	for (i = 0; i < nPols; i++) lwgeom_free((LWGEOM *) gv[i].geom);
 	rtdealloc(gv);
@@ -369,35 +324,29 @@ static void test_gdal_polygonize() {
 	}
 	*/
 
-#ifdef GDALFPOLYGONIZE
 	CU_ASSERT_DOUBLE_EQUAL(gv[0].val, 1.8, FLT_EPSILON);
-#else
-	CU_ASSERT_DOUBLE_EQUAL(gv[0].val, 2.0, 1.);
-#endif
-
-	wkt = lwgeom_to_text((const LWGEOM *) gv[0].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((3 1,3 2,2 2,2 3,1 3,1 6,2 6,2 7,3 7,3 8,5 8,5 6,3 6,3 3,4 3,5 3,5 1,3 1))");
-	rtdealloc(wkt);
+	gobserved = (LWGEOM *)gv[0].geom;
+	gexpected = lwgeom_from_wkt(
+	    "POLYGON((3 1,3 2,2 2,2 3,1 3,1 6,2 6,2 7,3 7,3 8,5 8,5 6,3 6,3 3,4 3,5 3,5 1,3 1))", LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
 	CU_ASSERT_DOUBLE_EQUAL(gv[1].val, 0.0, FLT_EPSILON);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[1].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((3 3,3 6,6 6,6 3,3 3))");
-	rtdealloc(wkt);
+	gobserved = (LWGEOM *)gv[1].geom;
+	gexpected = lwgeom_from_wkt("POLYGON((3 3,3 6,6 6,6 3,3 3))", LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
-#ifdef GDALFPOLYGONIZE
 	CU_ASSERT_DOUBLE_EQUAL(gv[2].val, 2.8, FLT_EPSILON);
-#else
-	CU_ASSERT_DOUBLE_EQUAL(gv[2].val, 3.0, 1.);
-#endif
-
-	wkt = lwgeom_to_text((const LWGEOM *) gv[2].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((5 1,5 3,6 3,6 6,5 6,5 8,6 8,6 7,7 7,7 6,8 6,8 3,7 3,7 2,6 2,6 1,5 1))");
-	rtdealloc(wkt);
+	gobserved = (LWGEOM *)gv[2].geom;
+	gexpected = lwgeom_from_wkt("POLYGON((5 1,5 3,6 3,6 6,5 6,5 8,6 8,6 7,7 7,7 6,8 6,8 3,7 3,7 2,6 2,6 1,5 1))",
+				    LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
 	CU_ASSERT_DOUBLE_EQUAL(gv[3].val, 0.0, FLT_EPSILON);
-	wkt = lwgeom_to_text((const LWGEOM *) gv[3].geom);
-	CU_ASSERT_STRING_EQUAL(wkt, "POLYGON((0 0,0 9,9 9,9 0,0 0),(6 7,6 8,3 8,3 7,2 7,2 6,1 6,1 3,2 3,2 2,3 2,3 1,6 1,6 2,7 2,7 3,8 3,8 6,7 6,7 7,6 7))");
-	rtdealloc(wkt);
+	gobserved = (LWGEOM *)gv[3].geom;
+	gexpected = lwgeom_from_wkt(
+	    "POLYGON((0 0,0 9,9 9,9 0,0 0),(6 7,6 8,3 8,3 7,2 7,2 6,1 6,1 3,2 3,2 2,3 2,3 1,6 1,6 2,7 2,7 3,8 3,8 6,7 6,7 7,6 7))",
+	    LW_PARSER_CHECK_NONE);
+	CU_ASSERT_DOUBLE_EQUAL(lwgeom_area(gobserved), lwgeom_area(gexpected), FLT_EPSILON);
 
 	for (i = 0; i < nPols; i++) lwgeom_free((LWGEOM *) gv[i].geom);
 	rtdealloc(gv);
@@ -471,6 +420,9 @@ static void test_raster_to_gdal() {
 
 	if (gdal) CPLFree(gdal);
 
+	gdal = rt_raster_to_gdal(raster, srs, "PCIDSK", NULL, &gdalSize);
+	CU_ASSERT(gdal == NULL);
+
 	cu_free_raster(raster);
 }
 
@@ -510,8 +462,8 @@ static void test_gdal_to_raster() {
 	CU_ASSERT(gddrv != NULL);
 	CU_ASSERT_EQUAL(destroy, 0);
 	CU_ASSERT(gdds != NULL);
-	CU_ASSERT_EQUAL(GDALGetRasterXSize(gdds), width);
-	CU_ASSERT_EQUAL(GDALGetRasterYSize(gdds), height);
+	CU_ASSERT_EQUAL((uint32_t)GDALGetRasterXSize(gdds), width);
+	CU_ASSERT_EQUAL((uint32_t)GDALGetRasterYSize(gdds), height);
 
 	rast = rt_raster_from_gdal_dataset(gdds);
 	CU_ASSERT(rast != NULL);
@@ -556,8 +508,8 @@ static void test_gdal_to_raster() {
 	CU_ASSERT(gddrv != NULL);
 	CU_ASSERT_EQUAL(destroy, 0);
 	CU_ASSERT(gdds != NULL);
-	CU_ASSERT_EQUAL(GDALGetRasterXSize(gdds), width);
-	CU_ASSERT_EQUAL(GDALGetRasterYSize(gdds), height);
+	CU_ASSERT_EQUAL((uint32_t)GDALGetRasterXSize(gdds), width);
+	CU_ASSERT_EQUAL((uint32_t)GDALGetRasterYSize(gdds), height);
 
 	rast = rt_raster_from_gdal_dataset(gdds);
 	CU_ASSERT(rast != NULL);
@@ -576,6 +528,8 @@ static void test_gdal_to_raster() {
 	}
 
 	GDALClose(gdds);
+	gdds = NULL;
+	gddrv = NULL;
 
 	cu_free_raster(rast);
 	cu_free_raster(raster);
@@ -642,14 +596,16 @@ static void test_gdal_warp() {
 }
 
 /* register tests */
-CU_TestInfo gdal_tests[] = {
-	PG_TEST(test_gdal_configured),
-	PG_TEST(test_gdal_drivers),
-	PG_TEST(test_gdal_rasterize),
-	PG_TEST(test_gdal_polygonize),
-	PG_TEST(test_raster_to_gdal),
-	PG_TEST(test_gdal_to_raster),
-	PG_TEST(test_gdal_warp),
-	CU_TEST_INFO_NULL
-};
-CU_SuiteInfo gdal_suite = {"gdal",  NULL,  NULL, gdal_tests};
+void gdal_suite_setup(void);
+void gdal_suite_setup(void)
+{
+	CU_pSuite suite = CU_add_suite("gdal", NULL, NULL);
+	PG_ADD_TEST(suite, test_gdal_configured);
+	PG_ADD_TEST(suite, test_gdal_drivers);
+	PG_ADD_TEST(suite, test_gdal_rasterize);
+	PG_ADD_TEST(suite, test_gdal_polygonize);
+	PG_ADD_TEST(suite, test_raster_to_gdal);
+	PG_ADD_TEST(suite, test_gdal_to_raster);
+	PG_ADD_TEST(suite, test_gdal_warp);
+}
+
