@@ -1,5 +1,18 @@
 #!/usr/bin/perl
 
+#
+# PostGIS - Spatial Types for PostgreSQL
+# http://postgis.net
+#
+# Copyright (C) 2012-2014 Sandro Santilli <strk@kbt.io>
+# Copyright (C) 2014-2015 Regina Obe <lr@pcorp.us>
+# Copyright (C) 2012-2013 Paul Ramsey <pramsey@cleverelephant.ca>
+#
+# This is free software; you can redistribute and/or modify it under
+# the terms of the GNU General Public Licence. See the COPYING file.
+#
+
+#$| = 1;
 use File::Basename;
 use File::Temp 'tempdir';
 #use File::Which;
@@ -9,28 +22,29 @@ use Cwd 'abs_path';
 use Getopt::Long;
 use strict;
 
+##################################################################
+# Add . to @INC if removed for CVE-2016-1238
+##################################################################
+BEGIN {
+	push @INC, "." if(!grep /^\.$/, @INC);
+}
+
 
 ##################################################################
 # Usage ./run_test.pl <testname> [<testname>]
 #
-#  Create the spatial database 'postgis_reg' (or whatever $DB 
+#  Create the spatial database 'postgis_reg' (or whatever $DB
 #  is set to) if it doesn't already exist.
 #
 #  Run the <testname>.sql script
 #  Check output against <testname>_expected
 ##################################################################
 
-if ( @ARGV < 1 )
-{
-	usage();
-}
-
-
 ##################################################################
 # Global configuration items
 ##################################################################
 
-our $DB = "postgis_reg";
+our $DB = $ENV{"POSTGIS_REGRESS_DB"} || "postgis_reg";
 our $REGDIR = abs_path(dirname($0));
 our $SHP2PGSQL = $REGDIR . "/../loader/shp2pgsql";
 our $PGSQL2SHP = $REGDIR . "/../loader/pgsql2shp";
@@ -45,26 +59,62 @@ my $OPT_CLEAN = 0;
 my $OPT_NODROP = 0;
 my $OPT_NOCREATE = 0;
 my $OPT_UPGRADE = 0;
+my $OPT_DUMPRESTORE = 0;
 my $OPT_WITH_TOPO = 0;
 my $OPT_WITH_RASTER = 0;
 my $OPT_WITH_SFCGAL = 0;
 my $OPT_EXPECT = 0;
 my $OPT_EXTENSIONS = 0;
+my $OPT_EXTVERSION = '';
+my $OPT_UPGRADE_PATH = '';
+our $OPT_UPGRADE_FROM = '';
+my $OPT_UPGRADE_TO = '';
 my $VERBOSE = 0;
+my $OPT_SCHEMA = 'public';
 
 GetOptions (
 	'verbose' => \$VERBOSE,
 	'clean' => \$OPT_CLEAN,
-	'nodrop' => \$OPT_NODROP, 
+	'nodrop' => \$OPT_NODROP,
 	'upgrade' => \$OPT_UPGRADE,
+	'upgrade-path=s' => \$OPT_UPGRADE_PATH,
+	'dumprestore' => \$OPT_DUMPRESTORE,
 	'nocreate' => \$OPT_NOCREATE,
 	'topology' => \$OPT_WITH_TOPO,
 	'raster' => \$OPT_WITH_RASTER,
 	'sfcgal' => \$OPT_WITH_SFCGAL,
 	'expect' => \$OPT_EXPECT,
-	'extensions' => \$OPT_EXTENSIONS
+	'extensions' => \$OPT_EXTENSIONS,
+	'schema=s' => \$OPT_SCHEMA
 	);
 
+if ( @ARGV < 1 )
+{
+	usage();
+}
+
+if ( $OPT_UPGRADE_PATH )
+{
+  if ( ! $OPT_EXTENSIONS )
+  {
+    die "--upgrade-path is only supported with --extension"
+  }
+  $OPT_UPGRADE = 1; # implied
+  my @path = split ('--', $OPT_UPGRADE_PATH);
+  $OPT_UPGRADE_FROM = $path[0]
+    || die "Malformed upgrade path, <from>--<to> expected, $OPT_UPGRADE_PATH given";
+  $OPT_UPGRADE_TO = $path[1]
+    || die "Malformed upgrade path, <from>--<to> expected, $OPT_UPGRADE_PATH given";
+  print "Upgrade path: ${OPT_UPGRADE_FROM} --> ${OPT_UPGRADE_TO}\n";
+}
+
+if ( $OPT_EXTENSIONS )
+{
+	$OPT_WITH_RASTER = 1; # implied
+}
+
+
+
 ##################################################################
 # Set the locale to "C" so error messages match
 # Save original locale to set back
@@ -76,7 +126,11 @@ $ENV{"LC_ALL"} = "C";
 $ENV{"LANG"} = "C";
 
 # Add locale info to the psql options
-my $PGOPTIONS = $ENV{"PGOPTIONS"} . " -c lc_messages=C -c client_min_messages=NOTICE";
+# Add pg12 precision suppression
+my $PGOPTIONS = $ENV{"PGOPTIONS"};
+$PGOPTIONS .= " -c lc_messages=C";
+$PGOPTIONS .= " -c client_min_messages=NOTICE";
+$PGOPTIONS .= " -c extra_float_digits=0";
 $ENV{"PGOPTIONS"} = $PGOPTIONS;
 
 # Bring the path info in
@@ -122,7 +176,7 @@ foreach my $exec ( ($SHP2PGSQL, $PGSQL2SHP) )
 		print "failed\n";
 		die "Unable to find $exec executable.\n";
 	}
-	
+
 }
 
 if ( $OPT_WITH_RASTER )
@@ -157,7 +211,7 @@ else
 	$TMPDIR = tempdir( CLEANUP => 0 );
 }
 
-mkdir $TMPDIR if ( ! -d $TMPDIR );
+mkpath $TMPDIR; # make sure tmp dir exists
 
 # Set log name
 my $REGRESS_LOG = "${TMPDIR}/regress_log";
@@ -170,7 +224,7 @@ print "TMPDIR is $TMPDIR\n";
 # Prepare the database
 ##################################################################
 
-my @dblist = grep(/$DB/, split(/\n/, `psql -l`));
+my @dblist = grep(/\b$DB\b/, split(/\n/, `psql -Xl`));
 my $dbcount = @dblist;
 
 if ( $dbcount == 0 )
@@ -210,17 +264,114 @@ if ( ! $libver )
 	exit(1);
 }
 
+sub semver_lessthan
+{
+  my ($a,$b) = @_;
+  my @acomp = split(/\./, $a);
+  my @bcomp = split(/\./, $b);
+	foreach my $ac (@acomp) {
+		my $bc = shift(@bcomp);
+		return 0 if not defined($bc); # $b has less components
+		return 1 if ( $ac < $bc ); # $a is less than $b
+		return 0 if ( $ac > $bc ); # $a is not less than $b
+	}
+	# $a is equal to $b so far, if $b has more elements,
+	# it is bigger
+	return @bcomp ? 1 : 0;
+}
+
+sub create_upgrade_test_objects
+{
+  # TODO: allow passing the "upgrade-init" script via commandline
+
+  my $query = "create table upgrade_test(g1 geometry, g2 geography";
+  $query .= ", r raster" if ( $OPT_WITH_RASTER );
+  $query .= ")";
+  my $ret = sql($query);
+  unless ( $ret =~ /^CREATE/ ) {
+    `dropdb $DB`;
+    print "\nSomething went wrong creating upgrade_test table: $ret.\n";
+    exit(1);
+  }
+
+  if ( $OPT_WITH_RASTER )
+  {
+    $query = "UPDATE upgrade_test SET r = ";
+    $query .= " ST_AddBand(ST_MakeEmptyRaster(10, 10, 1, 1, 2, 2, 0, 0,4326), 1, '8BSI'::text, -129, NULL);";
+    $ret = sql($query);
+    unless ( $ret =~ /^UPDATE/ ) {
+      `dropdb $DB`;
+      print "\nSomething went wrong setting raster into upgrade_test table: $ret.\n";
+      exit(1);
+    }
+
+    $query = "set client_min_messages to error; select AddRasterConstraints('upgrade_test', 'r')";
+    $ret = sql($query);
+    unless ( $ret =~ /^t$/ ) {
+      `dropdb $DB`;
+      print "\nSomething went wrong adding raster constraints to upgrade_test: " . $ret . "\n";
+      exit(1);
+    }
+  }
+
+  if ( $OPT_WITH_TOPO )
+  {
+    $query = "select topology.createTopology('upgrade_test');";
+    $ret = sql($query);
+    unless ( $ret =~ /^[1-9][0-9]*$/ ) {
+      `dropdb $DB`;
+      print "\nSomething went wrong adding upgrade_test topology: " . $ret . "\n";
+      exit(1);
+    }
+  }
+}
+
+sub drop_upgrade_test_objects
+{
+  # TODO: allow passing the "upgrade-cleanup" script via commandline
+
+  my $ret = sql("drop table upgrade_test;");
+  unless ( $ret =~ /^DROP/ ) {
+    `dropdb $DB`;
+    print "\nSomething went wrong dropping spatial tables: $ret.\n";
+    exit(1);
+  }
+
+  if ( $OPT_WITH_TOPO )
+  {
+    my $query = "SELECT topology.DropTopology('upgrade_test');";
+    $ret = sql($query);
+    unless ( $ret =~ /^Topology 'upgrade_test' dropped$/ ) {
+      `dropdb $DB`;
+      print "\nSomething went wrong dropping upgrade_test topology: " . $ret . "\n";
+      exit(1);
+    }
+  }
+}
+
 
 if ( $OPT_UPGRADE )
 {
-	if ( $OPT_EXTENSIONS )
-	{	
-		upgrade_spatial_extensions();
-	}
-	else
-	{
+  create_upgrade_test_objects();
+
+  if ( $OPT_EXTENSIONS )
+  {
+    upgrade_spatial_extensions();
+  }
+  else
+  {
 	  upgrade_spatial();
   }
+
+  drop_upgrade_test_objects();
+
+  # Update libver
+  $libver = sql("select postgis_lib_version()");
+}
+
+if ( $OPT_DUMPRESTORE )
+{
+  dump_restore();
 }
 
 
@@ -235,9 +386,14 @@ my $libbuilddate = sql("select postgis_lib_build_date()");
 my $pgsqlver = sql("select version()");
 my $gdalver = sql("select postgis_gdal_version()") if $OPT_WITH_RASTER;
 my $sfcgalver = sql("select postgis_sfcgal_version()") if $OPT_WITH_SFCGAL;
+my $scriptver = sql("select postgis_scripts_installed()");
+my $raster_scriptver = sql("select postgis_raster_scripts_installed()")
+  if ( $OPT_WITH_RASTER );
 
 print "$pgsqlver\n";
 print "  Postgis $libver - r${svnrev} - $libbuilddate\n";
+print "  scripts ${scriptver}\n";
+print "  raster scripts ${raster_scriptver}\n" if ( $OPT_WITH_RASTER );
 print "  GEOS: $geosver\n" if $geosver;
 print "  PROJ: $projver\n" if $projver;
 print "  SFCGAL: $sfcgalver\n" if $sfcgalver;
@@ -265,13 +421,13 @@ foreach $TEST (@ARGV)
 
 	start_test($TEST);
 
-	# Check for a "-pre.pl" file in case there are setup commands 
+	# Check for a "-pre.pl" file in case there are setup commands
     eval_file("${TEST}-pre.pl");
 
 	# Check for a "-pre.sql" file in case there is setup SQL needed before
 	# the test can be run.
 	if ( -r "${TEST}-pre.sql" )
-	{	
+	{
 		run_simple_sql("${TEST}-pre.sql");
 		show_progress();
 	}
@@ -294,11 +450,16 @@ foreach $TEST (@ARGV)
 		my $rv = run_simple_test("${TEST}.sql", "${TEST}_expected");
 		pass() if $rv;
 	}
+	elsif ( -r "${TEST}.dmp" )
+	{
+		pass() if run_dumper_test();
+	}
 	else
 	{
-		print " skipped (can't read ${TEST}.sql)\n";
+		print " skipped (can't read any ${TEST}.{sql,dbf,tif,dmp})\n";
 		$SKIP++;
-		next;
+		# Even though we skipped this test, we will still run the cleanup
+		# scripts
 	}
 
 	if ( -r "${TEST}-post.sql" )
@@ -309,18 +470,18 @@ foreach $TEST (@ARGV)
 			print " ... but cleanup sql failed!";
 		}
 	}
-	
-	# Check for a "-post.pl" file in case there are teardown commands 
+
+	# Check for a "-post.pl" file in case there are teardown commands
     eval_file("${TEST}-post.pl");
-	
+
 }
 
 
-################################################################### 
+###################################################################
 # Uninstall postgis (serves as an uninstall test)
 ##################################################################
 
-# We only test uninstall if we've been asked to drop 
+# We only test uninstall if we've been asked to drop
 # and we did create
 # and nobody requested raster or topology
 # (until they have an uninstall script themself)
@@ -364,20 +525,23 @@ exit($FAIL);
 # Utility functions
 #
 
-sub usage 
+sub usage
 {
 	die qq{
-Usage: $0 <testname> [<testname>]
+Usage: $0 [<options>] <testname> [<testname>]
 Options:
-  --verbose    be verbose about failures
-  --nocreate   do not create the regression database on start
-  --upgrade    source the upgrade scripts on start
-  --nodrop     do not drop the regression database on exit
-  --raster     load also raster extension
-  --topology   load also topology extension
-  --sfcgal     use also sfcgal backend
-  --clean      cleanup test logs on exit
-  --expect     save obtained output as expected
+  -v, --verbose   be verbose about failures
+  --nocreate      do not create the regression database on start
+  --upgrade       source the upgrade scripts on start
+  --upgrade-path  upgrade path, format <from>--<to>
+  --dumprestore   dump and restore spatially-enabled db before running tests
+  --nodrop        do not drop the regression database on exit
+  --raster        load also raster extension
+  --topology      load also topology extension
+  --sfcgal        use also sfcgal backend
+  --clean         cleanup test logs on exit
+  --expect        save obtained output as expected
+  --extension     load using extensions
 };
 
 }
@@ -404,7 +568,7 @@ sub show_progress
 	print ".";
 }
 
-# pass <msg> 
+# pass <msg>
 sub pass
 {
   my $msg = shift;
@@ -438,10 +602,10 @@ sub fail
 	$FAIL++;
 }
 
-  
+
 
 ##################################################################
-# run_simple_sql 
+# run_simple_sql
 #   Run an sql script and hide results unless it fails.
 #   SQL input file name is $1
 ##################################################################
@@ -449,7 +613,7 @@ sub run_simple_sql
 {
 	my $sql = shift;
 
-	if ( ! -r $sql ) 
+	if ( ! -r $sql )
 	{
 		fail("can't read $sql");
 		return 0;
@@ -458,20 +622,21 @@ sub run_simple_sql
 	# Dump output to a temp file.
 	my $tmpfile = sprintf("%s/test_%s_tmp", $TMPDIR, $RUN);
 	my $cmd = "psql -v \"VERBOSITY=terse\" -tXA $DB < $sql > $tmpfile 2>&1";
+	#print($cmd);
 	my $rv = system($cmd);
-
 	# Check if psql errored out.
-	if ( $rv != 0 ) 
+	if ( $rv != 0 )
 	{
 		fail("Unable to run sql script $sql", $tmpfile);
 		return 0;
 	}
-	
+
 	# Check for ERROR lines
 	open FILE, "$tmpfile";
 	my @lines = <FILE>;
+	close FILE;
 	my @errors = grep(/^ERROR/, @lines);
-	
+
 	if ( @errors > 0 )
 	{
 		fail("Errors while running sql script $sql", $tmpfile);
@@ -487,14 +652,14 @@ sub drop_table
 	my $tblname = shift;
 	my $cmd = "psql -tXA -d $DB -c \"DROP TABLE IF EXISTS $tblname\" >> $REGRESS_LOG 2>&1";
 	my $rv = system($cmd);
-	die "Could not run: $cmd\n" if $rv;	
+	die "Could not run: $cmd\n" if $rv;
 }
 
 sub sql
 {
 	my $sql = shift;
 	my $result = `psql -tXA -d $DB -c "$sql"`;
-	$result =~ s/\n$//;
+	$result =~ s/[\n\r]*$//;
 	$result;
 }
 
@@ -513,7 +678,7 @@ sub eval_file
 }
 
 ##################################################################
-# run_simple_test 
+# run_simple_test
 #   Run an sql script and compare results with the given expected output
 #   SQL input is ${TEST}.sql, expected output is {$TEST}_expected
 ##################################################################
@@ -528,7 +693,7 @@ sub run_simple_test
 		fail("can't read $sql");
 		return 0;
 	}
-	
+
 	if ( ! $OPT_EXPECT )
 	{
 		if ( ! -r "$expected" )
@@ -562,6 +727,8 @@ sub run_simple_test
 	my $cmd = "psql -v \"VERBOSITY=terse\""
           . " -v \"tmpfile='$tmpfile'\""
           . " -v \"scriptdir=$scriptdir\""
+          . " -v \"regdir=$REGDIR\""
+          . " -v \"schema=$OPT_SCHEMA.\""
           . " -tXA $DB < $sql > $outfile 2>&1";
 	my $rv = system($cmd);
 
@@ -572,10 +739,10 @@ sub run_simple_test
 
 	# Strip the lines we don't care about
 	@lines = grep(!/^\$/, @lines);
-	@lines = grep(!/^(INSERT|DELETE|UPDATE|SELECT)/, @lines);
+	@lines = grep(!/^(INSERT|DELETE|UPDATE|SELECT|COPY|DO)/, @lines);
 	@lines = grep(!/^(CONTEXT|RESET|ANALYZE)/, @lines);
-	@lines = grep(!/^(DROP|CREATE|VACUUM)/, @lines);
-	@lines = grep(!/^(LOG|SET|TRUNCATE)/, @lines);
+	@lines = grep(!/^(DROP|CREATE|ALTER|VACUUM)/, @lines);
+	@lines = grep(!/^(LOG|SET|TRUNCATE|DISCARD)/, @lines);
 	@lines = grep(!/^LINE \d/, @lines);
 	@lines = grep(!/^\s+$/, @lines);
 
@@ -588,11 +755,12 @@ sub run_simple_test
 		$lines[$i] =~ s/[eE]([+-])0+(\d+)/e$1$2/g;
 		$lines[$i] =~ s/Self-intersection .*/Self-intersection/;
 		$lines[$i] =~ s/^ROLLBACK/COMMIT/;
+		$lines[$i] =~ s/^psql.*(NOTICE|WARNING|ERROR):/\1:/g;
 	}
-	
+
 	# Write out output file
 	open(FILE, ">$outfile");
-	foreach my $l (@lines) 
+	foreach my $l (@lines)
 	{
 		print FILE $l;
 	}
@@ -600,10 +768,10 @@ sub run_simple_test
 
 	# Clean up interim stuff
 	#remove_tree($betmpdir);
-	
+
 	if ( $OPT_EXPECT )
 	{
-		print " expected\n";
+		print " (expected)";
 		copy($outfile, $expected);
 	}
 	else
@@ -623,7 +791,7 @@ sub run_simple_test
 			return 1;
 		}
 	}
-	
+
 	return 1;
 }
 
@@ -656,7 +824,7 @@ sub run_loader_and_check_output
 	my ( $cmd, $rv );
 	my $outfile = "${TMPDIR}/loader.out";
 	my $errfile = "${TMPDIR}/loader.err";
-	
+
 	# ON_ERROR_STOP is used by psql to return non-0 on an error
 	my $psql_opts = " --no-psqlrc --variable ON_ERROR_STOP=true";
 
@@ -684,7 +852,7 @@ sub run_loader_and_check_output
 				return 0;
 			}
 		}
-		
+
 		# Run the loader SQL script.
 		show_progress();
 		$cmd = "psql $psql_opts -f $outfile $DB > $errfile 2>&1";
@@ -726,12 +894,13 @@ sub run_dumper_and_check_output
 
 	my ($cmd, $rv);
 	my $errfile = "${TMPDIR}/dumper.err";
-	
-	if ( $run_always || -r $expected_shp_file ) 
+
+	if ( $run_always || -r $expected_shp_file )
 	{
 		show_progress();
 		$cmd = "${PGSQL2SHP} -f ${TMPDIR}/dumper $DB $tblname > $errfile 2>&1";
 		$rv = system($cmd);
+
 		if ( $rv )
 		{
 			fail("$description: dumping loaded table", $errfile);
@@ -739,9 +908,11 @@ sub run_dumper_and_check_output
 		}
 
 		# Compare with expected output if there is any.
+
 		if ( -r $expected_shp_file )
 		{
 			show_progress();
+
 			my $diff = diff($expected_shp_file,  "$TMPDIR/dumper.shp");
 			if ( $diff )
 			{
@@ -780,7 +951,7 @@ sub run_raster_loader_and_check_output
 	my $expected_select_results_file = shift;
 	my $loader_options = shift;
 	my $run_always = shift;
-	
+
 	# ON_ERROR_STOP is used by psql to return non-0 on an error
 	my $psql_opts="--no-psqlrc --variable ON_ERROR_STOP=true";
 
@@ -788,20 +959,20 @@ sub run_raster_loader_and_check_output
 	my $outfile = "${TMPDIR}/loader.out";
 	my $errfile = "${TMPDIR}/loader.err";
 
-	if ( $run_always || -r $expected_sql_file || -r $expected_select_results_file ) 
+	if ( $run_always || -r $expected_sql_file || -r $expected_select_results_file )
 	{
 		show_progress();
 
 		# Produce the output SQL file.
 		$cmd = "$RASTER2PGSQL $loader_options ${TEST}.tif $tblname > $outfile 2> $errfile";
 		$rv = system($cmd);
-		
+
 		if ( $rv )
 		{
 		    fail("$description: running raster2pgsql", $errfile);
 		    return 0;
 	    }
-	    
+
 	    if ( -r $expected_sql_file )
 	    {
 	        show_progress();
@@ -811,7 +982,7 @@ sub run_raster_loader_and_check_output
 				fail(" $description: actual SQL does not match expected.", "$outfile");
 				return 0;
 			}
-	        
+
         }
 
 		# Run the loader SQL script.
@@ -831,26 +1002,26 @@ sub run_raster_loader_and_check_output
     		return 0 if ( ! $rv );
     	}
 	}
-    	
+
     return 1;
 }
 
 
 
 ##################################################################
-#  run_loader_test 
+#  run_loader_test
 #
 #  Load a shapefile with different methods, create a 'select *' SQL
-#  test and run simple test with provided expected output. 
+#  test and run simple test with provided expected output.
 #
 #  SHP input is ${TEST}.shp, expected output is {$TEST}.expected
 ##################################################################
-sub run_loader_test 
+sub run_loader_test
 {
 	# See if there is a custom command-line options file
 	my $opts_file = "${TEST}.opts";
 	my $custom_opts="";
-	
+
 	if ( -r $opts_file )
 	{
 		open(FILE, $opts_file);
@@ -918,20 +1089,103 @@ sub run_loader_test
 		}
 		drop_table($tblname);
 	}
-	
+
+	return 1;
+}
+
+##################################################################
+#  run_dumper_test
+#
+#  Run dumper and compare output with various expectances
+#  test and run simple test with provided expected output.
+#
+# input is ${TEST}.dmp, where last line is considered to be the
+# [table|query] argument for pgsql2shp and all the previous lines,
+# if any are
+#
+##################################################################
+sub run_dumper_test
+{
+  my $dump_file  = "${TEST}.dmp";
+
+  # ON_ERROR_STOP is used by psql to return non-0 on an error
+  my $psql_opts="--no-psqlrc --variable ON_ERROR_STOP=true";
+
+  my $shpfile = "${TMPDIR}/dumper-" . basename(${TEST}) . "-shp";
+  my $outfile = "${TMPDIR}/dumper-" . basename(${TEST}) . ".out";
+  my $errfile = "${TMPDIR}/dumper-" . basename(${TEST}) . ".err";
+
+  # Produce the output SHP file.
+  open DUMPFILE, "$dump_file" or die "Cannot open dump file $dump_file\n";
+  sleep(1);
+  my @dumplines = <DUMPFILE>;
+  close DUMPFILE;
+  my $dumpstring = join '', @dumplines;
+  chop($dumpstring);
+  my @cmd = ("${PGSQL2SHP}", "-f", ${shpfile}, ${DB}, ${dumpstring});
+  open my $stdout_save, '>&', *STDOUT or die "Cannot dup stdout\n";
+  open my $stderr_save, '>&', *STDERR or die "Cannot dup stdout\n";
+  open STDOUT, ">${outfile}" or die "Cannot write to ${outfile}\n";
+  open STDERR, ">${errfile}" or die "Cannot write to ${errfile}\n";
+  my $rv = system(@cmd);
+  open STDERR, '>&', $stderr_save;
+  open STDOUT, '>&', $stdout_save;
+  #sleep(3);
+  show_progress();
+
+  if ( $rv )
+  {
+    fail("dumping", "$errfile");
+    return 0;
+  }
+
+  my $numtests = 0;
+  foreach my $ext ("shp","prj","dbf","shx") {
+    my $obtained = ${shpfile}.".".$ext;
+    my $expected = ${TEST}."_expected.".$ext;
+    if ( $OPT_EXPECT )
+    {
+      copy($obtained, $expected);
+    }
+    elsif ( -r ${expected} ) {
+      show_progress();
+      $numtests++;
+      my $diff = diff($expected,  $obtained);
+      if ( $diff )
+      {
+        my $diffile = sprintf("%s/dumper_test_%s_diff", $TMPDIR, $ext);
+        open(FILE, ">$diffile");
+        print FILE $diff;
+        close(FILE);
+        fail("diff expected obtained", $diffile);
+        return 0;
+      }
+    }
+  }
+
+  #show_progress();
+
+  if ( $OPT_EXPECT ) {
+    print " (expected)";
+  }
+  elsif ( ! $numtests ) {
+    fail("no expectances!");
+    return 0;
+  }
+
 	return 1;
 }
 
 
 ##################################################################
-#  run_raster_loader_test 
+#  run_raster_loader_test
 ##################################################################
 sub run_raster_loader_test
 {
 	# See if there is a custom command-line options file
 	my $opts_file = "${TEST}.opts";
 	my $custom_opts="";
-	
+
 	if ( -r $opts_file )
 	{
 		open(FILE, $opts_file);
@@ -949,9 +1203,9 @@ sub run_raster_loader_test
 	{
 		return 0;
 	}
-	
+
 	drop_table($tblname);
-	
+
 	return 1;
 }
 
@@ -962,15 +1216,15 @@ sub run_raster_loader_test
 sub count_db_objects
 {
 	my $count = sql("WITH counts as (
-		select count(*) from pg_type union all 
-		select count(*) from pg_proc union all 
+		select count(*) from pg_type union all
+		select count(*) from pg_proc union all
 		select count(*) from pg_cast union all
 		select count(*) from pg_aggregate union all
 		select count(*) from pg_operator union all
 		select count(*) from pg_opclass union all
 		select count(*) from pg_namespace
 			where nspname NOT LIKE 'pg_%' union all
-		select count(*) from pg_opfamily ) 
+		select count(*) from pg_opfamily )
 		select sum(count) from counts");
 
  	return $count;
@@ -980,20 +1234,26 @@ sub count_db_objects
 ##################################################################
 # Create the spatial database
 ##################################################################
-sub create_spatial 
+sub create_db
+{
+	my $cmd = "createdb --encoding=UTF-8 --template=template0 --lc-collate=C $DB > $REGRESS_LOG";
+	return system($cmd);
+}
+
+sub create_spatial
 {
 	my ($cmd, $rv);
 	print "Creating database '$DB' \n";
 
-	$cmd = "createdb --encoding=UTF-8 --template=template0 --lc-collate=C $DB > $REGRESS_LOG";
-	$rv = system($cmd);
+  $rv = create_db();
+
 	$cmd = "createlang plpgsql $DB >> $REGRESS_LOG 2>&1";
 	$rv = system($cmd);
 
 	# Count database objects before installing anything
 	$OBJ_COUNT_PRE = count_db_objects();
 
-	if ( $OPT_EXTENSIONS ) 
+	if ( $OPT_EXTENSIONS )
 	{
 		prepare_spatial_extensions();
 	}
@@ -1008,12 +1268,12 @@ sub load_sql_file
 {
 	my $file = shift;
 	my $strict = shift;
-	
+
 	if ( $strict && ! -e $file )
 	{
-		die "Unable to find $file\n"; 
+		die "Unable to find $file\n";
 	}
-	
+
 	if ( -e $file )
 	{
 		# ON_ERROR_STOP is used by psql to return non-0 on an error
@@ -1021,8 +1281,11 @@ sub load_sql_file
 		my $cmd = "psql $psql_opts -Xf $file $DB >> $REGRESS_LOG 2>&1";
 		print "  $file\n" if $VERBOSE;
 		my $rv = system($cmd);
-		die "\nError encountered loading $file, see $REGRESS_LOG for details\n\n"
-		    if $rv;
+		if ( $rv )
+		{
+		  fail "Error encountered loading $file", $REGRESS_LOG;
+		  exit 1
+		}
 	}
 	return 1;
 }
@@ -1030,24 +1293,70 @@ sub load_sql_file
 # Prepare the database for spatial operations (extension method)
 sub prepare_spatial_extensions
 {
-	print "Preparing db '${DB}' using 'CREATE EXTENSION'\n"; 
-
 	# ON_ERROR_STOP is used by psql to return non-0 on an error
 	my $psql_opts = "--no-psqlrc --variable ON_ERROR_STOP=true";
-    my $cmd = "psql $psql_opts -c \"CREATE EXTENSION postgis\" $DB >> $REGRESS_LOG 2>&1";
-    my $rv = system($cmd);
+	my $sql = "CREATE EXTENSION postgis";
+	if ( $OPT_UPGRADE_FROM ) {
+		if ( $OPT_UPGRADE_FROM eq "unpackaged" ) {
+			prepare_spatial();
+			return;
+		}
+		$sql .= " VERSION '" . $OPT_UPGRADE_FROM . "'";
+	}
 
-	die "\nError encountered creating EXTENSION POSTGIS, see $REGRESS_LOG for details\n\n"
-	    if $rv;
+	print "Preparing db '${DB}' using: ${sql}\n";
 
-    if ( $OPT_WITH_TOPO )
-    {
-        $cmd = "psql $psql_opts -c \"CREATE EXTENSION postgis_topology\" $DB >> $REGRESS_LOG 2>&1";
-        $rv = system($cmd);
-    	die "\nError encountered creating EXTENSION POSTGIS_TOPOLOGY, see $REGRESS_LOG for details\n\n"
-    	    if $rv;
-    }
-    return 1;
+	my $cmd = "psql $psql_opts -c \"". $sql . "\" $DB >> $REGRESS_LOG 2>&1";
+	my $rv = system($cmd);
+
+  if ( $rv ) {
+  	fail "Error encountered creating EXTENSION POSTGIS", $REGRESS_LOG;
+  	die;
+	}
+
+	if ( $OPT_WITH_TOPO )
+	{
+		my $sql = "CREATE EXTENSION postgis_topology";
+		if ( $OPT_UPGRADE_FROM ) {
+			$sql .= " VERSION '" . $OPT_UPGRADE_FROM . "'";
+		}
+
+		print "Preparing db '${DB}' using: ${sql}\n";
+
+ 		$cmd = "psql $psql_opts -c \"" . $sql . "\" $DB >> $REGRESS_LOG 2>&1";
+		$rv = system($cmd);
+  	if ( $rv ) {
+  		fail "Error encountered creating EXTENSION POSTGIS_TOPOLOGY", $REGRESS_LOG;
+  		die;
+		}
+ 	}
+
+	if ( $OPT_WITH_SFCGAL )
+	{
+		do {
+			my $sql = "CREATE EXTENSION postgis_sfcgal";
+			if ( $OPT_UPGRADE_FROM ) {
+				if ( semver_lessthan($OPT_UPGRADE_FROM, "2.2.0") )
+				{
+					print "NOTICE: skipping SFCGAL extension create "
+							. "as not available in version '$OPT_UPGRADE_FROM'\n";
+					last;
+				}
+				$sql .= " VERSION '" . $OPT_UPGRADE_FROM . "'";
+			}
+
+			print "Preparing db '${DB}' using: ${sql}\n";
+
+			$cmd = "psql $psql_opts -c \"" . $sql . "\" $DB >> $REGRESS_LOG 2>&1";
+			$rv = system($cmd);
+			if ( $rv ) {
+				fail "Error encountered creating EXTENSION POSTGIS_SFCGAL", $REGRESS_LOG;
+				die;
+			}
+		} while (0);
+	}
+
+ 	return 1;
 }
 
 # Prepare the database for spatial operations (old method)
@@ -1058,19 +1367,21 @@ sub prepare_spatial
 	# Load postgis.sql into the database
 	load_sql_file("${STAGED_SCRIPTS_DIR}/postgis.sql", 1);
 	load_sql_file("${STAGED_SCRIPTS_DIR}/postgis_comments.sql", 0);
-	
+	load_sql_file("${STAGED_SCRIPTS_DIR}/postgis_proc_set_search_path.sql", 0);
+
 	if ( $OPT_WITH_TOPO )
 	{
 		print "Loading Topology into '${DB}'\n";
 		load_sql_file("${STAGED_SCRIPTS_DIR}/topology.sql", 1);
 		load_sql_file("${STAGED_SCRIPTS_DIR}/topology_comments.sql", 0);
 	}
-	
+
 	if ( $OPT_WITH_RASTER )
 	{
 		print "Loading Raster into '${DB}'\n";
 		load_sql_file("${STAGED_SCRIPTS_DIR}/rtpostgis.sql", 1);
 		load_sql_file("${STAGED_SCRIPTS_DIR}/raster_comments.sql", 0);
+		load_sql_file("${STAGED_SCRIPTS_DIR}/rtpostgis_proc_set_search_path.sql", 0);
 	}
 
 	if ( $OPT_WITH_SFCGAL )
@@ -1088,7 +1399,7 @@ sub upgrade_spatial
 {
     print "Upgrading PostGIS in '${DB}' \n" ;
 
-    my $script = `ls ${STAGED_SCRIPTS_DIR}/postgis_upgrade_*_minor.sql`;
+    my $script = `ls ${STAGED_SCRIPTS_DIR}/postgis_upgrade.sql`;
     chomp($script);
 
     if ( -e $script )
@@ -1100,10 +1411,10 @@ sub upgrade_spatial
     {
         die "$script not found\n";
     }
-    
-    if ( $OPT_WITH_TOPO ) 
+
+    if ( $OPT_WITH_TOPO )
     {
-        my $script = `ls ${STAGED_SCRIPTS_DIR}/topology_upgrade_*_minor.sql`;
+        my $script = `ls ${STAGED_SCRIPTS_DIR}/topology_upgrade.sql`;
         chomp($script);
         if ( -e $script )
         {
@@ -1115,10 +1426,10 @@ sub upgrade_spatial
             die "$script not found\n";
         }
     }
-    
-    if ( $OPT_WITH_RASTER ) 
+
+    if ( $OPT_WITH_RASTER )
     {
-        my $script = `ls ${STAGED_SCRIPTS_DIR}/rtpostgis_upgrade_*_minor.sql`;
+        my $script = `ls ${STAGED_SCRIPTS_DIR}/rtpostgis_upgrade.sql`;
         chomp($script);
         if ( -e $script )
         {
@@ -1130,29 +1441,93 @@ sub upgrade_spatial
             die "$script not found\n";
         }
     }
+
+    if ( $OPT_WITH_SFCGAL )
+    {
+        my $script = `ls ${STAGED_SCRIPTS_DIR}/sfcgal_upgrade.sql`;
+        chomp($script);
+        if ( -e $script )
+        {
+            print "Upgrading sfcgal\n";
+            load_sql_file($script);
+        }
+        else
+        {
+            die "$script not found\n";
+        }
+    }
+
     return 1;
 }
 
 # Upgrade an existing database (soft upgrade, extension method)
 sub upgrade_spatial_extensions
 {
-    print "Upgrading PostGIS in '${DB}' using 'ALTER EXTENSION'\n" ;
-
     # ON_ERROR_STOP is used by psql to return non-0 on an error
     my $psql_opts = "--no-psqlrc --variable ON_ERROR_STOP=true";
-    my $cmd = "psql $psql_opts -c \"ALTER EXTENSION postgis UPDATE TO '${libver}next'\" $DB >> $REGRESS_LOG 2>&1";
-    my $rv = system($cmd);
-    die "\nError encountered altering EXTENSION POSTGIS, see $REGRESS_LOG for details\n\n"
-	    if $rv;
+    my $nextver = $OPT_UPGRADE_TO ? "${OPT_UPGRADE_TO}" : "${libver}next";
+    my $sql = "ALTER EXTENSION postgis UPDATE TO '${nextver}'";
 
-    if ( $OPT_WITH_TOPO ) 
-    {
-      my $cmd = "psql $psql_opts -c \"ALTER EXTENSION postgis_topology UPDATE TO '${libver}next'\" $DB >> $REGRESS_LOG 2>&1";
-      my $rv = system($cmd);
-      die "\nError encountered altering EXTENSION POSTGIS_TOPOLOGY, see $REGRESS_LOG for details\n\n"
-        if $rv;
+    if ( $OPT_UPGRADE_FROM eq "unpackaged" ) {
+      $sql = "CREATE EXTENSION postgis VERSION '${nextver}' FROM unpackaged";
     }
-    
+
+    print "Upgrading PostGIS in '${DB}' using: ${sql}\n" ;
+
+    my $cmd = "psql $psql_opts -c \"" . $sql . "\" $DB >> $REGRESS_LOG 2>&1";
+    #print "CMD: " . $cmd . "\n";
+    my $rv = system($cmd);
+    if ( $rv ) {
+      fail "Error encountered altering EXTENSION POSTGIS", $REGRESS_LOG;
+      die;
+    }
+
+    if ( $OPT_WITH_TOPO )
+    {
+      my $sql = "ALTER EXTENSION postgis_topology UPDATE TO '${nextver}'";
+
+			if ( $OPT_UPGRADE_FROM eq "unpackaged" ) {
+				$sql = "CREATE EXTENSION postgis_topology VERSION '${nextver}' FROM unpackaged";
+			}
+
+			print "Upgrading PostGIS Topology in '${DB}' using: ${sql}\n" ;
+
+      my $cmd = "psql $psql_opts -c \"" . $sql . "\" $DB >> $REGRESS_LOG 2>&1";
+      my $rv = system($cmd);
+      if ( $rv ) {
+        fail "Error encountered altering EXTENSION POSTGIS_TOPOLOGY", $REGRESS_LOG;
+        die;
+      }
+    }
+
+    if ( $OPT_WITH_SFCGAL )
+    {
+			my $sql;
+
+			if ( $OPT_UPGRADE_FROM eq "unpackaged" ) {
+				$sql = "CREATE EXTENSION postgis_sfcgal VERSION '${nextver}' FROM unpackaged";
+			}
+			elsif ( $OPT_UPGRADE_FROM && semver_lessthan($OPT_UPGRADE_FROM, "2.2.0") )
+			{
+				print "NOTICE: installing SFCGAL extension on upgrade "
+						. "as it was not available in version '$OPT_UPGRADE_FROM'\n";
+				$sql = "CREATE EXTENSION postgis_sfcgal VERSION '${nextver}'";
+			}
+			else
+			{
+				$sql = "ALTER EXTENSION postgis_sfcgal UPDATE TO '${nextver}'";
+			}
+      $cmd = "psql $psql_opts -c \"" . $sql . "\" $DB >> $REGRESS_LOG 2>&1";
+
+			print "Upgrading PostGIS SFCGAL in '${DB}' using: ${sql}\n" ;
+
+      $rv = system($cmd);
+      if ( $rv ) {
+        fail "Error encountered creating EXTENSION POSTGIS_SFCGAL", $REGRESS_LOG;
+        die;
+      }
+    }
+
     return 1;
 }
 
@@ -1181,16 +1556,26 @@ sub drop_spatial_extensions
 {
     # ON_ERROR_STOP is used by psql to return non-0 on an error
     my $psql_opts="--no-psqlrc --variable ON_ERROR_STOP=true";
-    my $ok = 1; 
+    my $ok = 1;
     my ($cmd, $rv);
-    
+
     if ( $OPT_WITH_TOPO )
     {
-        $cmd = "psql $psql_opts -c \"DROP EXTENSION postgis_topology\" $DB >> $REGRESS_LOG 2>&1";
+        # NOTE: "manually" dropping topology schema as EXTENSION does not
+        #       take care of that itself, see
+        #       http://trac.osgeo.org/postgis/ticket/2138
+        $cmd = "psql $psql_opts -c \"DROP EXTENSION postgis_topology; DROP SCHEMA topology;\" $DB >> $REGRESS_LOG 2>&1";
         $rv = system($cmd);
       	$ok = 0 if $rv;
     }
-    
+
+    if ( $OPT_WITH_SFCGAL )
+    {
+        $cmd = "psql $psql_opts -c \"DROP EXTENSION postgis_sfcgal;\" $DB >> $REGRESS_LOG 2>&1";
+        $rv = system($cmd);
+        $ok = 0 if $rv;
+    }
+
     $cmd = "psql $psql_opts -c \"DROP EXTENSION postgis\" $DB >> $REGRESS_LOG 2>&1";
     $rv = system($cmd);
   	die "\nError encountered dropping EXTENSION POSTGIS, see $REGRESS_LOG for details\n\n"
@@ -1204,9 +1589,9 @@ sub uninstall_spatial
 {
 	my $ok;
 	start_test("uninstall");
-	
+
 	if ( $OPT_EXTENSIONS )
-	{	
+	{
 		$ok = drop_spatial_extensions();
 	}
 	else
@@ -1214,11 +1599,11 @@ sub uninstall_spatial
 		$ok = drop_spatial();
 	}
 
-	if ( $ok ) 
+	if ( $ok )
 	{
 		show_progress(); # on to objects count
 		$OBJ_COUNT_POST = count_db_objects();
-		
+
 		if ( $OBJ_COUNT_POST != $OBJ_COUNT_PRE )
 		{
 			fail("Object count pre-install ($OBJ_COUNT_PRE) != post-uninstall ($OBJ_COUNT_POST)");
@@ -1226,13 +1611,63 @@ sub uninstall_spatial
 		}
 		else
 		{
-            pass("($OBJ_COUNT_PRE)");
+			pass("($OBJ_COUNT_PRE)");
 			return 1;
 		}
 	}
-	
+
 	return 0;
-}  
+}
+
+# Dump and restore the database
+sub dump_restore
+{
+  my $DBDUMP = $TMPDIR . '/' . $DB . ".dump";
+  my $rv;
+
+	print "Dumping and restoring database '${DB}'\n";
+
+  $rv = system("pg_dump -Fc -f${DBDUMP} ${DB} >> $REGRESS_LOG 2>&1");
+  if ( $rv ) {
+    fail("Could not dump ${DB}", $REGRESS_LOG);
+		die;
+  }
+
+  $rv = system("dropdb ${DB} >> $REGRESS_LOG 2>&1");
+  if ( $rv ) {
+    fail("Could not drop ${DB}", $REGRESS_LOG);
+		die;
+  }
+
+  $rv = create_db();
+  if ( $rv ) {
+    fail("Could not create ${DB}", $REGRESS_LOG);
+		die;
+  }
+
+  $rv = system("pg_restore -d ${DB} ${DBDUMP} >> $REGRESS_LOG 2>&1");
+  if ( $rv ) {
+    fail("Could not restore ${DB}", $REGRESS_LOG);
+    die;
+  }
+
+  if ( $OPT_WITH_TOPO )
+  {
+    # We need to re-add "topology" to the search_path as it is lost
+    # on dump/reload, see https://trac.osgeo.org/postgis/ticket/3454
+    my $psql_opts = "--no-psqlrc --variable ON_ERROR_STOP=true";
+    my $cmd = "psql $psql_opts -c \"SELECT topology.AddToSearchPath('topology')\" $DB >> $REGRESS_LOG 2>&1";
+    $rv = system($cmd);
+    if ( $rv ) {
+      fail("Error encountered adding topology to search path after restore", $REGRESS_LOG);
+      die;
+    }
+  }
+
+  unlink($DBDUMP);
+
+  return 1;
+}
 
 sub diff
 {
@@ -1240,12 +1675,12 @@ sub diff
 	my $diffstr = '';
 
 	if ( $sysdiff ) {
-		$diffstr = `diff --strip-trailing-cr -u $expected_file $obtained_file`;
+		$diffstr = `diff --strip-trailing-cr -u $expected_file $obtained_file 2>&1`;
 		return $diffstr;
 	}
 
-	open(OBT, $obtained_file) || die "Cannot open $obtained_file\n";
-	open(EXP, $expected_file) || die "Cannot open $expected_file\n";
+	open(OBT, $obtained_file) || return "Cannot open $obtained_file\n";
+	open(EXP, $expected_file) || return "Cannot open $expected_file\n";
 	my $lineno = 0;
 	while (!eof(OBT) or !eof(EXP)) {
 		# TODO: check for premature end of one or the other ?
